#include "problem.h"

#include <meshagent.h>
#include "stdloop.h"

using namespace Gascoigne;


class LocalLoop : public StdLoop
{
public:

  void BasicInit(const ParamFile& paramfile,
                 const ProblemContainer* PC,
                 const FunctionalContainer* FC)
  {
    //GetMeshAgentPointer() = new MyMeshAgent;
    StdLoop::BasicInit(paramfile, PC, FC);
  }

  void run(const std::string& problemlabel)
  {

    Vector u("u"), f("f") ;
    Matrix A("A");

    // READ PARAM FILE
    int niter;
    DataFormatHandler DFH;
    DFH.insert("niter", &niter, 1);
    FileScanner FS(DFH, _paramfile, "Loop");


    // REFINEMENT LOOP
    for (_iter = 1; _iter <= niter; _iter++) {


      GetMultiLevelSolver()->ReInit();
      GetMultiLevelSolver()->SetProblem(problemlabel);
      GetMultiLevelSolver()->ReInitMatrix(A);
      GetMultiLevelSolver()->ReInitVector(u);
      GetMultiLevelSolver()->ReInitVector(f);
      GetSolverInfos()->GetNLInfo().control().matrixmustbebuild() = 1;

      Solve(A,u,f);

      //Problem 3.3
      //Compute Functionals
      //DoubleVector juh = Functionals(u,f);
      //Output
      //std::cout << "meshsize h : " << pow(2, -_iter) << std::endl;
      //std::cout << "Functional : " << pow(juh[0],.5) << std::endl;


      if (_iter < niter) {
        GetMeshAgent()->global_refine(1);
      }

    }

  }


};


class H1norm : public virtual DomainFunctional {

  std::string GetName() const { return "H1"; }

  MyProblemData data;

public:

  H1norm(const MyProblemData& dat) : data(dat)
  {
  }


  double J(const FemFunction& U, const Vertex2d& v) const
  {

    //Problem 3.3
    //Here implement the exact solution and subsrtact it from FemFunction

    //doube u =
    //doube dudx =
    //doube dudy =


    //this return H1 norm of solution you need change to the norm of error!
    return U[0].m()*U[0].m() + U[0].x()*U[0].x() + U[0].y()*U[0].y() ;
  }

};


int main(int argc, char **argv)
{
  ParamFile paramfile("gascoigne.param");
  if(argc>=2) {
    paramfile.SetName(argv[1]);
  }

  MyProblemDescriptor MPD;
  MPD.BasicInit(paramfile);

  ProblemContainer PC;
  PC.AddProblem("Exact Problem", &MPD);

  FunctionalContainer FC;

  //Problem 3.3 and 3.5
  /*
    H1norm H1(MPD.GetData());
    L2norm L2(MPD.GetData());

    FC.AddFunctional("H1",&H1);
    FC.AddFunctional("L2",&L2);
  */

  //Problem 3.2
  StdLoop loop;

  loop.BasicInit(paramfile, &PC, &FC);
  loop.run("Exact Problem");

  return 0;
}
