#ifndef PROBLEM_H
#define PROBLEM_H

#include <dirichletdata.h>
#include <domainrighthandside.h>
#include <laplace.h>
#include <meshagent.h>
#include <problemdescriptorbase.h>

#include <cmath>
#define pi M_PI

using namespace Gascoigne;

// This class collects all the data that is taken from the parameter file
// and can be used in different classes such as Equation, DirichletData or
// RightHandSide. New data, such as required in Problem 3.5 must be put here.
class MyProblemData : public Gascoigne::LaplaceData
{

public:
  void BasicInit(const ParamFile & pf)
  {
    Gascoigne::LaplaceData::BasicInit(pf);
  }
};



// class for specifying Dirichlet data
class MyDirichletData : public DirichletData
{
  MyProblemData data;

public:

  MyDirichletData(const Gascoigne::ParamFile& pf,
                  const MyProblemData& dat) : DirichletData(pf), data(dat)
  {}

  std::string GetName() const { return "Dirichlet Data"; }

  void operator()(DoubleVector &b, const Vertex2d &v, int col) const
  {
    b.zero();
  }
};


// class for specifying problem's right hand side
class MyRightHandSide : public DomainRightHandSide
{

  MyProblemData data;

public:

  MyRightHandSide(const MyProblemData& dat) : data(dat)
  {
  }


  // the clone-Function to create a new RightHandSide object on the fly.
  // The new objects gets the ProblemData object so that all parameters
  // are available
  MyRightHandSide* createNew() const
  {
    return new MyRightHandSide(data);
  }


  int GetNcomp() const { return 1; }
  std::string GetName() const { return "Right Hand Side"; }

  double operator()(int c, const Vertex2d &v) const
  {
    // TODO 3.1 : Implement the right hand side!
    return 0;
   }
};




// main class for defining the problem to solve
class MyProblemDescriptor : public ProblemDescriptorBase {

  MyProblemData data;

public:

  std::string GetName() const {  return "Exact Problem";}
  MyProblemData GetData() const { return data; }

  void BasicInit(const ParamFile & pf)
  {
    GetParamFile() = pf;

    // Initialize the Problem Data
    data.BasicInit(pf) ;

    
    // equation to solve
    GetEquationPointer() = new Laplace2d(data);

    // definition for right hand side
    GetRightHandSidePointer() = new MyRightHandSide(data);

    // definition of dirichlet boundary data
    GetDirichletDataPointer() = new MyDirichletData(pf, data);


    ProblemDescriptorBase::BasicInit(pf);
  }


};


#endif /* PROBLEM_H */
